"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
const ts = require("typescript");
const index_1 = require("../models/index");
const nodes_1 = require("./utils/nodes");
const converter_events_1 = require("./converter-events");
/**
 * The context describes the current state the converter is in.
 * @internal
 */
class Context {
    /**
     * Create a new Context instance.
     *
     * @param converter  The converter instance that has created the context.
     * @param entryPoints  A list of all entry points for this project.
     * @param checker  The TypeChecker instance returned by the TypeScript compiler.
     * @internal
     */
    constructor(converter, checker, program, project = new index_1.ProjectReflection(converter.name), scope = project) {
        this.converter = converter;
        this.checker = checker;
        this.program = program;
        this.project = project;
        this.scope = scope;
    }
    /** @internal */
    get logger() {
        return this.converter.application.logger;
    }
    /**
     * Return the compiler options.
     */
    getCompilerOptions() {
        return this.converter.application.options.getCompilerOptions();
    }
    /**
     * Return the type declaration of the given node.
     *
     * @param node  The TypeScript node whose type should be resolved.
     * @returns The type declaration of the given node.
     */
    getTypeAtLocation(node) {
        let nodeType;
        try {
            nodeType = this.checker.getTypeAtLocation(node);
        }
        catch (_a) {
            // ignore
        }
        if (!nodeType) {
            if (node.symbol) {
                nodeType = this.checker.getDeclaredTypeOfSymbol(node.symbol);
            }
            else if (node.parent && node.parent.symbol) {
                nodeType = this.checker.getDeclaredTypeOfSymbol(node.parent.symbol);
            }
            else if (node.parent &&
                node.parent.parent &&
                node.parent.parent.symbol) {
                nodeType = this.checker.getDeclaredTypeOfSymbol(node.parent.parent.symbol);
            }
        }
        return nodeType;
    }
    getSymbolAtLocation(node) {
        let symbol = this.checker.getSymbolAtLocation(node);
        if (!symbol && nodes_1.isNamedNode(node)) {
            symbol = this.checker.getSymbolAtLocation(node.name);
        }
        return symbol;
    }
    expectSymbolAtLocation(node) {
        const symbol = this.getSymbolAtLocation(node);
        if (!symbol) {
            const { line } = ts.getLineAndCharacterOfPosition(node.getSourceFile(), node.pos);
            throw new Error(`Expected a symbol for node with kind ${ts.SyntaxKind[node.kind]} at ${node.getSourceFile().fileName}:${line + 1}`);
        }
        return symbol;
    }
    resolveAliasedSymbol(symbol) {
        while (symbol && ts.SymbolFlags.Alias & symbol.flags) {
            symbol = this.checker.getAliasedSymbol(symbol);
        }
        return symbol;
    }
    createDeclarationReflection(kind, symbol, name = getHumanName(symbol.name)) {
        const reflection = new index_1.DeclarationReflection(name, kind, this.scope);
        this.addChild(reflection);
        if (this.converter.isExternal(symbol)) {
            reflection.setFlag(index_1.ReflectionFlag.External);
        }
        this.registerReflection(reflection, symbol);
        this.converter.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, this, reflection, 
        // FIXME this isn't good enough.
        this.converter.getNodesForSymbol(symbol, kind)[0]);
        return reflection;
    }
    addChild(reflection) {
        var _a;
        var _b;
        if (this.scope instanceof index_1.ContainerReflection) {
            (_a = (_b = this.scope).children) !== null && _a !== void 0 ? _a : (_b.children = []);
            this.scope.children.push(reflection);
        }
    }
    shouldIgnore(symbol) {
        return this.converter.shouldIgnore(symbol, this.checker);
    }
    /**
     * Register a newly generated reflection. All created reflections should be
     * passed to this method to ensure that the project helper functions work correctly.
     *
     * @param reflection  The reflection that should be registered.
     * @param symbol  The symbol the given reflection was resolved from.
     */
    registerReflection(reflection, symbol) {
        this.project.registerReflection(reflection, symbol);
    }
    /**
     * Trigger a node reflection event.
     *
     * All events are dispatched on the current converter instance.
     *
     * @param name  The name of the event that should be triggered.
     * @param reflection  The triggering reflection.
     * @param node  The triggering TypeScript node if available.
     */
    trigger(name, reflection, node) {
        this.converter.trigger(name, this, reflection, node);
    }
    /**
     * @param callback  The callback function that should be executed with the changed context.
     */
    withScope(scope) {
        const context = new Context(this.converter, this.checker, this.program, this.project, scope);
        return context;
    }
}
exports.Context = Context;
const builtInSymbolRegExp = /^__@(\w+)$/;
function getHumanName(name) {
    const match = builtInSymbolRegExp.exec(name);
    if (match) {
        return `[Symbol.${match[1]}]`;
    }
    return name;
}
//# sourceMappingURL=context.js.map