"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var Converter_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Converter = void 0;
const ts = require("typescript");
const assert = require("assert");
const index_1 = require("../models/index");
const context_1 = require("./context");
const components_1 = require("./components");
const component_1 = require("../utils/component");
const utils_1 = require("../utils");
const types_1 = require("./types");
const converter_events_1 = require("./converter-events");
const symbols_1 = require("./symbols");
const path_1 = require("path");
const fs_1 = require("../utils/fs");
const paths_1 = require("../utils/paths");
const enum_1 = require("../utils/enum");
/**
 * Compiles source files using TypeScript and converts compiler symbols to reflections.
 */
let Converter = Converter_1 = class Converter extends component_1.ChildableComponent {
    /**
     * Compile the given source files and create a project reflection for them.
     *
     * @param entryPoints the entry points of this program.
     * @param program the program to document that has already been type checked.
     */
    convert(entryPoints, program) {
        this.externalPatternCache = void 0;
        const checker = program.getTypeChecker();
        const context = new context_1.Context(this, checker, program);
        this.trigger(Converter_1.EVENT_BEGIN, context);
        this.compile(program, entryPoints, context);
        const project = this.resolve(context);
        // This should only do anything if a plugin does something bad.
        project.removeDanglingReferences();
        this.trigger(Converter_1.EVENT_END, context);
        return project;
    }
    /**
     * Convert the given TypeScript type into its TypeDoc type reflection.
     *
     * @param context  The context object describing the current state the converter is in.
     * @param referenceTarget The target to be used to attempt to resolve reference types
     * @returns The TypeDoc type reflection representing the given node and type.
     */
    convertType(context, node, referenceTarget) {
        return types_1.convertType(context, node, referenceTarget);
    }
    getNodesForSymbol(symbol, kind) {
        var _a;
        const wantedKinds = {
            [index_1.ReflectionKind.Project]: [ts.SyntaxKind.SourceFile],
            [index_1.ReflectionKind.Module]: [ts.SyntaxKind.SourceFile],
            [index_1.ReflectionKind.Namespace]: [
                ts.SyntaxKind.ModuleDeclaration,
                ts.SyntaxKind.SourceFile,
            ],
            [index_1.ReflectionKind.Enum]: [ts.SyntaxKind.EnumDeclaration],
            [index_1.ReflectionKind.EnumMember]: [ts.SyntaxKind.EnumMember],
            [index_1.ReflectionKind.Variable]: [ts.SyntaxKind.VariableDeclaration],
            [index_1.ReflectionKind.Function]: [
                ts.SyntaxKind.FunctionDeclaration,
                // Tricky: We need this because we pretend some variables are functions
                ts.SyntaxKind.VariableDeclaration,
            ],
            [index_1.ReflectionKind.Class]: [ts.SyntaxKind.ClassDeclaration],
            [index_1.ReflectionKind.Interface]: [ts.SyntaxKind.InterfaceDeclaration],
            [index_1.ReflectionKind.Constructor]: [ts.SyntaxKind.Constructor],
            [index_1.ReflectionKind.Property]: [
                ts.SyntaxKind.PropertyDeclaration,
                ts.SyntaxKind.PropertySignature,
            ],
            [index_1.ReflectionKind.Method]: [
                ts.SyntaxKind.MethodDeclaration,
                // Tricky: We need this because some properties are treated as methods
                ts.SyntaxKind.PropertyDeclaration,
                ts.SyntaxKind.PropertySignature,
            ],
            [index_1.ReflectionKind.CallSignature]: [ts.SyntaxKind.CallSignature],
            [index_1.ReflectionKind.IndexSignature]: [ts.SyntaxKind.IndexSignature],
            [index_1.ReflectionKind.ConstructorSignature]: [
                ts.SyntaxKind.ConstructSignature,
            ],
            [index_1.ReflectionKind.Parameter]: [ts.SyntaxKind.Parameter],
            [index_1.ReflectionKind.TypeLiteral]: [ts.SyntaxKind.TypeLiteral],
            [index_1.ReflectionKind.TypeParameter]: [ts.SyntaxKind.TypeParameter],
            [index_1.ReflectionKind.Accessor]: [
                ts.SyntaxKind.GetAccessor,
                ts.SyntaxKind.SetAccessor,
            ],
            [index_1.ReflectionKind.GetSignature]: [ts.SyntaxKind.GetAccessor],
            [index_1.ReflectionKind.SetSignature]: [ts.SyntaxKind.SetAccessor],
            [index_1.ReflectionKind.ObjectLiteral]: [
                ts.SyntaxKind.ObjectLiteralExpression,
            ],
            [index_1.ReflectionKind.TypeAlias]: [ts.SyntaxKind.TypeAliasDeclaration],
            [index_1.ReflectionKind.Event]: [],
            [index_1.ReflectionKind.Reference]: [
                ts.SyntaxKind.NamespaceExport,
                ts.SyntaxKind.ExportSpecifier,
            ],
        }[kind];
        const declarations = (_a = symbol.getDeclarations()) !== null && _a !== void 0 ? _a : [];
        return declarations.filter((d) => wantedKinds.includes(d.kind));
    }
    /**
     * Compile the files within the given context and convert the compiler symbols to reflections.
     *
     * @param context  The context object describing the current state the converter is in.
     * @returns An array containing all errors generated by the TypeScript compiler.
     */
    compile(program, entryPoints, context) {
        const baseDir = fs_1.getCommonDirectory(entryPoints);
        const needsSecondPass = [];
        for (const entry of entryPoints) {
            const sourceFile = program.getSourceFile(utils_1.normalizePath(entry));
            if (!sourceFile) {
                this.application.logger.warn(`Unable to locate entry point: ${entry}`);
                continue;
            }
            needsSecondPass.push(sourceFile);
            this.convertExports(context, sourceFile, entryPoints, baseDir);
        }
        for (const file of needsSecondPass) {
            this.convertReExports(context, file);
        }
    }
    convertExports(context, node, entryPoints, baseDir) {
        var _a;
        const symbol = (_a = context.checker.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
        let moduleContext;
        if (entryPoints.length === 1) {
            // Special case for when we're giving a single entry point, we don't need to
            // create modules for each entry. Register the project as this module.
            context.project.registerReflection(context.project, symbol);
            moduleContext = context;
        }
        else if (symbol) {
            const reflection = context.createDeclarationReflection(index_1.ReflectionKind.Module, symbol, getModuleName(node.fileName, baseDir));
            moduleContext = context.withScope(reflection);
        }
        else {
            this.application.logger.warn(`If specifying a global file as an entry point, only one entry point may be specified. (${node.fileName})`);
            return;
        }
        for (const exp of getExports(context, node).filter((exp) => { var _a; return (_a = context
            .resolveAliasedSymbol(exp)
            .getDeclarations()) === null || _a === void 0 ? void 0 : _a.every((d) => d.getSourceFile() === node.getSourceFile()); })) {
            symbols_1.convertSymbol(moduleContext, exp);
        }
    }
    convertReExports(context, node) {
        var _a;
        const symbol = (_a = context.checker.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
        // Was a global "module"... no re exports.
        if (symbol == null)
            return;
        const moduleReflection = context.project.getReflectionFromSymbol(symbol);
        assert(moduleReflection instanceof index_1.ContainerReflection ||
            moduleReflection instanceof index_1.DeclarationReflection);
        const moduleContext = context.withScope(moduleReflection);
        for (const exp of getExports(context, node).filter((exp) => { var _a; return (_a = context
            .resolveAliasedSymbol(exp)
            .getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => d.getSourceFile() !== node.getSourceFile()); })) {
            symbols_1.convertSymbol(moduleContext, exp);
        }
    }
    /**
     * Resolve the project within the given context.
     *
     * @param context  The context object describing the current state the converter is in.
     * @returns The final project reflection.
     */
    resolve(context) {
        this.trigger(Converter_1.EVENT_RESOLVE_BEGIN, context);
        const project = context.project;
        for (const reflection of Object.values(project.reflections)) {
            this.trigger(Converter_1.EVENT_RESOLVE, context, reflection);
        }
        this.trigger(Converter_1.EVENT_RESOLVE_END, context);
        return project;
    }
    /** @internal */
    shouldIgnore(symbol, checker) {
        if (this.excludeNotDocumented &&
            // If the enum is included, we should include members even if not documented.
            !enum_1.hasFlag(symbol.flags, ts.SymbolFlags.EnumMember) &&
            !symbol.getDocumentationComment(checker)) {
            return true;
        }
        if (!this.excludeExternals) {
            return false;
        }
        return this.isExternal(symbol);
    }
    /** @internal */
    isExternal(symbol) {
        var _a, _b;
        (_a = this.externalPatternCache) !== null && _a !== void 0 ? _a : (this.externalPatternCache = paths_1.createMinimatch(this.externalPattern));
        for (const node of (_b = symbol.getDeclarations()) !== null && _b !== void 0 ? _b : []) {
            if (this.externalPatternCache.some((p) => p.match(node.getSourceFile().fileName))) {
                return true;
            }
        }
        return false;
    }
};
/**
 * General events
 */
/**
 * Triggered when the converter begins converting a project.
 * The listener should implement [[IConverterCallback]].
 * @event
 */
Converter.EVENT_BEGIN = converter_events_1.ConverterEvents.BEGIN;
/**
 * Triggered when the converter has finished converting a project.
 * The listener should implement [[IConverterCallback]].
 * @event
 */
Converter.EVENT_END = converter_events_1.ConverterEvents.END;
/**
 * Factory events
 */
/**
 * Triggered when the converter has created a declaration reflection.
 * The listener should implement [[IConverterNodeCallback]].
 * @event
 */
Converter.EVENT_CREATE_DECLARATION = converter_events_1.ConverterEvents.CREATE_DECLARATION;
/**
 * Triggered when the converter has created a signature reflection.
 * The listener should implement [[IConverterNodeCallback]].
 * @event
 */
Converter.EVENT_CREATE_SIGNATURE = converter_events_1.ConverterEvents.CREATE_SIGNATURE;
/**
 * Triggered when the converter has created a parameter reflection.
 * The listener should implement [[IConverterNodeCallback]].
 * @event
 */
Converter.EVENT_CREATE_PARAMETER = converter_events_1.ConverterEvents.CREATE_PARAMETER;
/**
 * Triggered when the converter has created a type parameter reflection.
 * The listener should implement [[IConverterNodeCallback]].
 * @event
 */
Converter.EVENT_CREATE_TYPE_PARAMETER = converter_events_1.ConverterEvents.CREATE_TYPE_PARAMETER;
/**
 * Triggered when the converter has found a function implementation.
 * The listener should implement [[IConverterNodeCallback]].
 * @event
 */
Converter.EVENT_FUNCTION_IMPLEMENTATION = converter_events_1.ConverterEvents.FUNCTION_IMPLEMENTATION;
/**
 * Resolve events
 */
/**
 * Triggered when the converter begins resolving a project.
 * The listener should implement [[IConverterCallback]].
 * @event
 */
Converter.EVENT_RESOLVE_BEGIN = converter_events_1.ConverterEvents.RESOLVE_BEGIN;
/**
 * Triggered when the converter resolves a reflection.
 * The listener should implement [[IConverterResolveCallback]].
 * @event
 */
Converter.EVENT_RESOLVE = converter_events_1.ConverterEvents.RESOLVE;
/**
 * Triggered when the converter has finished resolving a project.
 * The listener should implement [[IConverterCallback]].
 * @event
 */
Converter.EVENT_RESOLVE_END = converter_events_1.ConverterEvents.RESOLVE_END;
__decorate([
    utils_1.BindOption("name")
], Converter.prototype, "name", void 0);
__decorate([
    utils_1.BindOption("externalPattern")
], Converter.prototype, "externalPattern", void 0);
__decorate([
    utils_1.BindOption("excludeExternals")
], Converter.prototype, "excludeExternals", void 0);
__decorate([
    utils_1.BindOption("excludeNotDocumented")
], Converter.prototype, "excludeNotDocumented", void 0);
__decorate([
    utils_1.BindOption("excludePrivate")
], Converter.prototype, "excludePrivate", void 0);
__decorate([
    utils_1.BindOption("excludeProtected")
], Converter.prototype, "excludeProtected", void 0);
Converter = Converter_1 = __decorate([
    component_1.Component({
        name: "converter",
        internal: true,
        childClass: components_1.ConverterComponent,
    })
], Converter);
exports.Converter = Converter;
function getModuleName(fileName, baseDir) {
    return utils_1.normalizePath(path_1.relative(baseDir, fileName)).replace(/(\.d)?\.[tj]sx?$/, "");
}
function getExports(context, node) {
    var _a, _b;
    let symbol = (_a = context.checker.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
    if (!symbol && ts.isModuleBlock(node)) {
        symbol = context.checker.getSymbolAtLocation(node.parent.name);
    }
    // The generated docs aren't great, but you really ought not be using
    // this in the first place... so it's better than nothing.
    const exportEq = (_b = symbol === null || symbol === void 0 ? void 0 : symbol.exports) === null || _b === void 0 ? void 0 : _b.get("export=");
    if (exportEq) {
        return [exportEq];
    }
    if (symbol) {
        return context.checker.getExportsOfModule(symbol);
    }
    // This is a global file, get all symbols declared in this file...
    // this isn't the best solution, it would be nice to have all globals given to a special
    // "globals" file, but this is uncommon enough that I'm skipping it for now.
    const sourceFile = node.getSourceFile();
    return context.checker
        .getSymbolsInScope(node, ts.SymbolFlags.ModuleMember)
        .filter((s) => { var _a; return (_a = s.getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => d.getSourceFile() === sourceFile); });
}
//# sourceMappingURL=converter.js.map