"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertSymbol = void 0;
const assert = require("assert");
const ts = require("typescript");
const models_1 = require("../models");
const array_1 = require("../utils/array");
const enum_1 = require("../utils/enum");
const convert_expression_1 = require("./convert-expression");
const converter_events_1 = require("./converter-events");
const signature_1 = require("./factories/signature");
// TODO: Index signatures
function getSymbolExportsWithFlag(symbol, flag) {
    var _a;
    const childSymbols = [];
    (_a = symbol.exports) === null || _a === void 0 ? void 0 : _a.forEach((child) => {
        if (child.flags & flag) {
            childSymbols.push(child);
        }
    });
    return childSymbols;
}
const symbolConverters = {
    [ts.SymbolFlags.RegularEnum]: convertEnum,
    [ts.SymbolFlags.ConstEnum]: convertEnum,
    [ts.SymbolFlags.EnumMember]: convertEnumMember,
    [ts.SymbolFlags.ValueModule]: convertNamespace,
    [ts.SymbolFlags.NamespaceModule]: convertNamespace,
    [ts.SymbolFlags.TypeAlias]: convertTypeAlias,
    [ts.SymbolFlags.Function]: convertFunctionOrMethod,
    [ts.SymbolFlags.Method]: convertFunctionOrMethod,
    [ts.SymbolFlags.Interface]: convertClassOrInterface,
    [ts.SymbolFlags.Property]: convertProperty,
    [ts.SymbolFlags.Class]: convertClassOrInterface,
    [ts.SymbolFlags.Constructor]: convertConstructor,
    [ts.SymbolFlags.Alias]: convertAlias,
    [ts.SymbolFlags.BlockScopedVariable]: convertVariable,
    [ts.SymbolFlags.FunctionScopedVariable]: convertVariable,
    [ts.SymbolFlags.GetAccessor]: convertAccessor,
    [ts.SymbolFlags.SetAccessor]: convertAccessor,
};
// Sanity check, if this fails a dev messed up.
for (const key of Object.keys(symbolConverters)) {
    if (!Number.isInteger(Math.log2(+key))) {
        throw new Error(`Symbol converter for key ${ts.SymbolFlags[+key]} does not specify a valid flag value.`);
    }
}
function convertSymbol(context, symbol, nameOverride) {
    var _a;
    if (context.shouldIgnore(symbol)) {
        return;
    }
    let flags = enum_1.removeFlag(symbol.flags, ts.SymbolFlags.Transient |
        ts.SymbolFlags.Assignment |
        ts.SymbolFlags.Optional |
        ts.SymbolFlags.Prototype);
    // Declaration merging - the only type (excluding enum/enum, ns/ns, etc)
    // that TD supports is merging a class and interface. All others are
    // represented as multiple reflections
    if (enum_1.hasFlag(symbol.flags, ts.SymbolFlags.Class)) {
        flags = enum_1.removeFlag(flags, ts.SymbolFlags.Interface);
    }
    // Kind of declaration merging... we treat this as a property with get/set signatures.
    if (enum_1.hasFlag(symbol.flags, ts.SymbolFlags.GetAccessor)) {
        flags = enum_1.removeFlag(flags, ts.SymbolFlags.SetAccessor);
    }
    for (const flag of enum_1.getEnumFlags(flags)) {
        if (!(flag in symbolConverters)) {
            context.logger.verbose(`Missing converter for symbol: ${symbol.name} with flag ${ts.SymbolFlags[flag]}`);
        }
        (_a = symbolConverters[flag]) === null || _a === void 0 ? void 0 : _a.call(symbolConverters, context, symbol, nameOverride);
    }
}
exports.convertSymbol = convertSymbol;
function convertSymbols(context, symbols) {
    for (const symbol of symbols) {
        convertSymbol(context, symbol);
    }
}
function convertEnum(context, symbol, nameOverride) {
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Enum, symbol, nameOverride);
    if (symbol.flags & ts.SymbolFlags.ConstEnum) {
        reflection.setFlag(models_1.ReflectionFlag.Const);
    }
    convertSymbols(context.withScope(reflection), getSymbolExportsWithFlag(symbol, ts.SymbolFlags.EnumMember));
}
function convertEnumMember(context, symbol, nameOverride) {
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.EnumMember, symbol, nameOverride);
    reflection.defaultValue = JSON.stringify(context.checker.getConstantValue(symbol.getDeclarations()[0]));
}
function convertNamespace(context, symbol, nameOverride) {
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Namespace, symbol, nameOverride);
    convertSymbols(context.withScope(reflection), getSymbolExportsWithFlag(symbol, ts.SymbolFlags.ModuleMember));
}
function convertTypeAlias(context, symbol, nameOverride) {
    var _a, _b;
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.TypeAlias, symbol, nameOverride);
    const declaration = (_a = symbol === null || symbol === void 0 ? void 0 : symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.find(ts.isTypeAliasDeclaration);
    assert(declaration);
    reflection.type = context.converter.convertType(context, declaration.type, declaration);
    reflection.typeParameters = (_b = declaration.typeParameters) === null || _b === void 0 ? void 0 : _b.map((param) => createTypeParamReflection(param, context.withScope(reflection), declaration));
}
function createTypeParamReflection(param, context, typeContextNode) {
    const constraint = param.constraint
        ? context.converter.convertType(context, param.constraint, typeContextNode)
        : void 0;
    const defaultType = param.default
        ? context.converter.convertType(context, param.default, typeContextNode)
        : void 0;
    const paramRefl = new models_1.TypeParameterReflection(param.name.text, constraint, defaultType, context.scope);
    context.registerReflection(paramRefl, undefined);
    context.trigger(converter_events_1.ConverterEvents.CREATE_TYPE_PARAMETER, paramRefl, param);
    return paramRefl;
}
function convertFunctionOrMethod(context, symbol, nameOverride) {
    var _a, _b, _c, _d;
    // Can't just check method flag because this might be called for properties as well
    // This will *NOT* be called for variables that look like functions, they need a special case.
    const isMethod = !!(symbol.flags &
        (ts.SymbolFlags.Property | ts.SymbolFlags.Method));
    const allDeclarations = (_b = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.filter(ts.isFunctionLike)) !== null && _b !== void 0 ? _b : [];
    // Don't do anything if we inherited this method and it is private.
    if (isMethod &&
        isInherited(context, symbol) &&
        allDeclarations.length > 0 &&
        enum_1.hasFlag(ts.getCombinedModifierFlags(allDeclarations[0]), ts.ModifierFlags.Private)) {
        return;
    }
    const reflection = context.createDeclarationReflection(isMethod ? models_1.ReflectionKind.Method : models_1.ReflectionKind.Function, symbol, nameOverride);
    // Note: We have to filter unique declarations here because TS might give us
    // the same declaration twice in mixins. This happens to method1 in the mixin
    // test when converting SomeClassWithMixin
    const declarations = array_1.unique(allDeclarations);
    if (declarations.length && isMethod) {
        // All method signatures must have the same modifier flags.
        setModifiers(declarations[0], reflection);
        const parentSymbol = context.project.getSymbolFromReflection(context.scope);
        assert(parentSymbol, "Tried to convert an arrow method without a parent.");
        if ((_c = parentSymbol
            .getDeclarations()) === null || _c === void 0 ? void 0 : _c.some((d) => d === declarations[0].parent)) {
            reflection.setOverwrites();
        }
    }
    const signatures = array_1.filterMap(declarations, (decl) => {
        if (!!decl.body && declarations.length > 1) {
            context.trigger(converter_events_1.ConverterEvents.FUNCTION_IMPLEMENTATION, reflection, decl);
            return;
        }
        return context.checker.getSignatureFromDeclaration(decl);
    });
    const scope = context.withScope(reflection);
    (_d = reflection.signatures) !== null && _d !== void 0 ? _d : (reflection.signatures = []);
    for (const [signature, declaration] of array_1.zip(signatures, declarations)) {
        const converted = signature_1.createSignature(scope, models_1.ReflectionKind.CallSignature, signature, declaration);
        reflection.signatures.push(converted);
    }
}
// getDeclaredTypeOfSymbol gets the INSTANCE type
// getTypeOfSymbolAtLocation gets the STATIC type
function convertClassOrInterface(context, symbol, nameOverride) {
    var _a, _b, _c;
    const reflection = context.createDeclarationReflection(ts.SymbolFlags.Class & symbol.flags
        ? models_1.ReflectionKind.Class
        : models_1.ReflectionKind.Interface, symbol, nameOverride);
    const reflectionContext = context.withScope(reflection);
    const instanceType = context.checker.getDeclaredTypeOfSymbol(symbol);
    assert(instanceType.isClassOrInterface());
    const classDeclaration = (_a = symbol
        .getDeclarations()) === null || _a === void 0 ? void 0 : _a.find(ts.isClassDeclaration);
    if (classDeclaration) {
        setModifiers(classDeclaration, reflection);
        // Classes can have static props
        const staticType = context.checker.getTypeOfSymbolAtLocation(symbol, classDeclaration);
        for (const prop of context.checker.getPropertiesOfType(staticType)) {
            // Don't convert namespace members, or the prototype here.
            if (prop.flags &
                (ts.SymbolFlags.ModuleMember | ts.SymbolFlags.Prototype))
                continue;
            convertSymbol(reflectionContext, prop);
        }
        const constructMember = new models_1.DeclarationReflection("constructor", models_1.ReflectionKind.Constructor, reflection);
        reflectionContext.addChild(constructMember);
        // The symbol is already taken by the class.
        context.registerReflection(constructMember, undefined);
        context.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, constructMember, classDeclaration.getChildren().find(ts.isConstructorDeclaration));
        const constructContext = reflectionContext.withScope(constructMember);
        constructMember.signatures = staticType
            .getConstructSignatures()
            .map((sig) => {
            const sigRef = signature_1.createSignature(constructContext, models_1.ReflectionKind.ConstructorSignature, sig);
            sigRef.name = `new ${reflectionContext.scope.name}`; // eww.. preserving legacy behavior.
            sigRef.originalName = sigRef.name;
            return sigRef;
        });
    }
    // Classes/interfaces usually just have properties...
    convertSymbols(reflectionContext, context.checker.getPropertiesOfType(instanceType));
    // And type arguments
    if (instanceType.typeParameters) {
        reflection.typeParameters = instanceType.typeParameters.map((param) => {
            var _a, _b, _c;
            const declaration = (_b = (_a = param.symbol) === null || _a === void 0 ? void 0 : _a.declarations) === null || _b === void 0 ? void 0 : _b[0];
            assert(ts.isTypeParameterDeclaration(declaration));
            return createTypeParamReflection(declaration, reflectionContext, (_c = symbol.getDeclarations()) === null || _c === void 0 ? void 0 : _c[0]);
        });
    }
    // We also might do some inheritance
    const declarations = (_c = (_b = symbol
        .getDeclarations()) === null || _b === void 0 ? void 0 : _b.filter((d) => ts.isInterfaceDeclaration(d) || ts.isClassDeclaration(d))) !== null && _c !== void 0 ? _c : [];
    const extendedTypes = array_1.flatMap(declarations, (decl) => {
        var _a;
        return array_1.flatMap((_a = decl.heritageClauses) !== null && _a !== void 0 ? _a : [], (clause) => {
            if (clause.token !== ts.SyntaxKind.ExtendsKeyword) {
                return [];
            }
            return clause.types.map((type) => context.converter.convertType(reflectionContext, type, decl));
        });
    });
    if (extendedTypes.length) {
        reflection.extendedTypes = array_1.uniqueByEquals(extendedTypes);
    }
    const implementedTypes = array_1.flatMap(declarations, (decl) => {
        var _a;
        return array_1.flatMap((_a = decl.heritageClauses) !== null && _a !== void 0 ? _a : [], (clause) => {
            if (clause.token !== ts.SyntaxKind.ImplementsKeyword) {
                return [];
            }
            return clause.types.map((type) => context.converter.convertType(reflectionContext, type, decl));
        });
    });
    if (implementedTypes.length) {
        reflection.implementedTypes = array_1.uniqueByEquals(implementedTypes);
    }
    // Interfaces might also have call signatures
    // Classes might too, because of declaration merging
    const callSignatures = context.checker
        .getSignaturesOfType(instanceType, ts.SignatureKind.Call)
        .map((sig) => signature_1.createSignature(reflectionContext, models_1.ReflectionKind.CallSignature, sig));
    if (callSignatures.length) {
        reflection.signatures = callSignatures;
    }
    // We also might have constructor signatures
    // This is potentially a problem with classes having
    convertConstructSignatures(reflectionContext, symbol);
}
function convertProperty(context, symbol, nameOverride) {
    var _a, _b, _c;
    const declarations = (_a = symbol.getDeclarations()) !== null && _a !== void 0 ? _a : [];
    const parentSymbol = context.project.getSymbolFromReflection(context.scope);
    assert(parentSymbol);
    // Don't do anything if we inherited this property and it is private.
    if (isInherited(context, symbol) &&
        declarations.length > 0 &&
        enum_1.hasFlag(ts.getCombinedModifierFlags(declarations[0]), ts.ModifierFlags.Private)) {
        return;
    }
    // Special case: We pretend properties are methods if they look like methods.
    // This happens with mixins / weird inheritance.
    if (declarations.every((decl) => ts.isMethodSignature(decl) || ts.isMethodDeclaration(decl))) {
        return convertFunctionOrMethod(context, symbol, nameOverride);
    }
    // Special case: "arrow methods" should be treated as methods.
    if (declarations.length === 1) {
        const declaration = declarations[0];
        if (ts.isPropertyDeclaration(declaration) &&
            !declaration.type &&
            declaration.initializer &&
            ts.isArrowFunction(declaration.initializer)) {
            return convertArrowAsMethod(context, symbol, declaration.initializer, nameOverride);
        }
    }
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Property, symbol, nameOverride);
    const declaration = (_b = symbol.getDeclarations()) === null || _b === void 0 ? void 0 : _b[0];
    let parameterType;
    if (declaration &&
        (ts.isPropertyDeclaration(declaration) ||
            ts.isPropertySignature(declaration) ||
            ts.isParameter(declaration))) {
        parameterType = declaration.type;
        setModifiers(declaration, reflection);
        const parentSymbol = context.project.getSymbolFromReflection(context.scope);
        assert(parentSymbol, "Tried to convert a property without a parent.");
        if ((_c = parentSymbol
            .getDeclarations()) === null || _c === void 0 ? void 0 : _c.some((d) => d === declaration.parent)) {
            reflection.setOverwrites();
        }
        if (ts.isPrivateIdentifier(declaration.name)) {
            reflection.setFlag(models_1.ReflectionFlag.Private);
        }
    }
    reflection.defaultValue = declaration && convert_expression_1.convertDefaultValue(declaration);
    // FIXME: This is a horrible hack because getTypeOfSymbol is not exposed.
    // The right solution here is probably to keep track of parent nodes...
    // but that's tricky because not every reflection is guaranteed to have a
    // parent node. This will probably break in a future TS version.
    reflection.type = context.converter.convertType(context, parameterType !== null && parameterType !== void 0 ? parameterType : context.checker.getTypeOfSymbolAtLocation(symbol, {}), declaration);
}
function convertArrowAsMethod(context, symbol, arrow, nameOverride) {
    var _a;
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Method, symbol, nameOverride);
    const rc = context.withScope(reflection);
    const signature = context.checker.getSignatureFromDeclaration(arrow);
    assert(signature);
    const parentSymbol = context.project.getSymbolFromReflection(context.scope);
    assert(parentSymbol, "Tried to convert an arrow method without a parent.");
    if ((_a = parentSymbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => d === arrow.parent)) {
        reflection.setOverwrites();
    }
    reflection.signatures = [
        signature_1.createSignature(rc, models_1.ReflectionKind.CallSignature, signature, arrow),
    ];
}
function convertConstructor(context, symbol) {
    var _a, _b;
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Constructor, symbol, "constructor");
    const reflectionContext = context.withScope(reflection);
    const declarations = (_b = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.filter(ts.isConstructorDeclaration)) !== null && _b !== void 0 ? _b : [];
    const signatures = declarations.map((decl) => {
        const sig = context.checker.getSignatureFromDeclaration(decl);
        assert(sig);
        return sig;
    });
    reflection.signatures = signatures.map((sig) => {
        const sigRef = signature_1.createSignature(reflectionContext, models_1.ReflectionKind.ConstructorSignature, sig);
        sigRef.name = `new ${context.scope.name}`; // eww.. preserving legacy behavior.
        sigRef.originalName = sigRef.name;
        return sigRef;
    });
}
function convertConstructSignatures(context, symbol) {
    const type = context.checker.getDeclaredTypeOfSymbol(symbol);
    // These get added as a "constructor" member of this interface. This is a problem... but nobody
    // has complained yet. We really ought to have a constructSignatures property on the reflection instead.
    const constructSignatures = context.checker.getSignaturesOfType(type, ts.SignatureKind.Construct);
    if (constructSignatures.length) {
        const constructMember = new models_1.DeclarationReflection("constructor", models_1.ReflectionKind.Constructor, context.scope);
        context.addChild(constructMember);
        context.registerReflection(constructMember, undefined);
        context.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, constructMember, 
        // FIXME this isn't good enough.
        context.converter.getNodesForSymbol(symbol, models_1.ReflectionKind.Constructor)[0]);
        const constructContext = context.withScope(constructMember);
        constructMember.signatures = constructSignatures.map((sig) => {
            const sigRef = signature_1.createSignature(constructContext, models_1.ReflectionKind.ConstructorSignature, sig);
            sigRef.name = `new ${context.scope.name}`; // eww.. preserving legacy behavior.
            sigRef.originalName = sigRef.name;
            return sigRef;
        });
    }
}
function convertAlias(context, symbol, nameOverride) {
    const reflection = context.project.getReflectionFromSymbol(context.resolveAliasedSymbol(symbol));
    if (!reflection) {
        // We don't have this, convert it.
        convertSymbol(context, context.resolveAliasedSymbol(symbol), nameOverride !== null && nameOverride !== void 0 ? nameOverride : symbol.name);
    }
    else {
        // We already have this. Create a reference.
        const ref = new models_1.ReferenceReflection(nameOverride !== null && nameOverride !== void 0 ? nameOverride : symbol.name, reflection, context.scope);
        context.addChild(ref);
        context.registerReflection(ref, symbol);
        context.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, reflection, 
        // FIXME this isn't good enough.
        context.converter.getNodesForSymbol(symbol, models_1.ReflectionKind.Reference)[0]);
    }
}
function convertVariable(context, symbol, nameOverride) {
    var _a;
    const declaration = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a[0];
    assert(declaration);
    const type = context.checker.getTypeOfSymbolAtLocation(symbol, declaration);
    if (type.getCallSignatures().length && !type.getProperties().length) {
        return convertVariableAsFunction(context, symbol, nameOverride);
    }
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Variable, symbol, nameOverride);
    let typeNode;
    if (ts.isVariableDeclaration(declaration)) {
        // Otherwise we might have destructuring
        typeNode = declaration.type;
    }
    reflection.type = context.converter.convertType(context.withScope(reflection), typeNode !== null && typeNode !== void 0 ? typeNode : context.checker.getTypeOfSymbolAtLocation(symbol, declaration), declaration);
    setModifiers(declaration, reflection);
    // Does anyone care about this? I doubt it...
    if (enum_1.hasFlag(symbol.flags, ts.SymbolFlags.BlockScopedVariable)) {
        reflection.setFlag(models_1.ReflectionFlag.Const, enum_1.hasFlag(declaration.parent.flags, ts.NodeFlags.Const));
    }
    reflection.defaultValue = convert_expression_1.convertDefaultValue(declaration);
}
function convertVariableAsFunction(context, symbol, nameOverride) {
    var _a, _b;
    const declaration = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a[0];
    assert(declaration);
    const type = context.checker.getTypeOfSymbolAtLocation(symbol, declaration);
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Function, symbol, nameOverride);
    setModifiers(declaration, reflection);
    // Does anyone care about this? I doubt it...
    if (enum_1.hasFlag(symbol.flags, ts.SymbolFlags.BlockScopedVariable)) {
        reflection.setFlag(models_1.ReflectionFlag.Const, enum_1.hasFlag(declaration.parent.flags, ts.NodeFlags.Const));
    }
    const reflectionContext = context.withScope(reflection);
    context.trigger(converter_events_1.ConverterEvents.FUNCTION_IMPLEMENTATION, reflection, declaration);
    (_b = reflection.signatures) !== null && _b !== void 0 ? _b : (reflection.signatures = []);
    for (const signature of type.getCallSignatures()) {
        reflection.signatures.push(signature_1.createSignature(reflectionContext, models_1.ReflectionKind.CallSignature, signature));
    }
}
function convertAccessor(context, symbol, nameOverride) {
    var _a, _b;
    const reflection = context.createDeclarationReflection(models_1.ReflectionKind.Accessor, symbol, nameOverride);
    const rc = context.withScope(reflection);
    const getDeclaration = (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.find(ts.isGetAccessor);
    if (getDeclaration) {
        const signature = context.checker.getSignatureFromDeclaration(getDeclaration);
        if (signature) {
            reflection.getSignature = signature_1.createSignature(rc, models_1.ReflectionKind.GetSignature, signature, getDeclaration);
        }
    }
    const setDeclaration = (_b = symbol.getDeclarations()) === null || _b === void 0 ? void 0 : _b.find(ts.isSetAccessor);
    if (setDeclaration) {
        const signature = context.checker.getSignatureFromDeclaration(setDeclaration);
        if (signature) {
            reflection.getSignature = signature_1.createSignature(rc, models_1.ReflectionKind.SetSignature, signature, setDeclaration);
        }
    }
}
function isInherited(context, symbol) {
    var _a;
    const parentSymbol = context.project.getSymbolFromReflection(context.scope);
    assert(parentSymbol);
    return (((_a = parentSymbol
        .getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d) => { var _a; return (_a = symbol.getDeclarations()) === null || _a === void 0 ? void 0 : _a.some((d2) => d2.parent === d); })) === false);
}
function setModifiers(declaration, reflection) {
    const modifiers = ts.getCombinedModifierFlags(declaration);
    // Note: We only set this flag if the modifier is present because we allow
    // fake "private" or "protected" members via @private and @protected
    if (enum_1.hasFlag(modifiers, ts.ModifierFlags.Private)) {
        reflection.setFlag(models_1.ReflectionFlag.Private);
    }
    if (enum_1.hasFlag(modifiers, ts.ModifierFlags.Protected)) {
        reflection.setFlag(models_1.ReflectionFlag.Protected);
    }
    if (enum_1.hasFlag(modifiers, ts.ModifierFlags.Public)) {
        reflection.setFlag(models_1.ReflectionFlag.Public);
    }
    reflection.setFlag(models_1.ReflectionFlag.Optional, !!declaration.questionToken);
    reflection.setFlag(models_1.ReflectionFlag.Readonly, enum_1.hasFlag(modifiers, ts.ModifierFlags.Readonly));
    reflection.setFlag(models_1.ReflectionFlag.Abstract, enum_1.hasFlag(modifiers, ts.ModifierFlags.Abstract));
    reflection.setFlag(models_1.ReflectionFlag.Static, enum_1.hasFlag(modifiers, ts.ModifierFlags.Static));
}
//# sourceMappingURL=symbols.js.map