"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertType = exports.loadConverters = void 0;
const assert = require("assert");
const ts = require("typescript");
const models_1 = require("../models");
const template_literal_1 = require("../models/types/template-literal");
const array_1 = require("../utils/array");
const converter_events_1 = require("./converter-events");
const signature_1 = require("./factories/signature");
const symbols_1 = require("./symbols");
const converters = new Map();
function loadConverters() {
    if (converters.size)
        return;
    for (const actor of [
        arrayConverter,
        conditionalConverter,
        exprWithTypeArgsConverter,
        indexedAccessConverter,
        inferredConverter,
        intersectionConverter,
        keywordConverter,
        parensConverter,
        predicateConverter,
        queryConverter,
        typeLiteralConverter,
        referenceConverter,
        namedTupleMemberConverter,
        mappedConverter,
        literalTypeConverter,
        templateLiteralConverter,
        thisConverter,
        tupleConverter,
        typeOperatorConverter,
        unionConverter,
    ]) {
        for (const key of actor.kind) {
            assert(!converters.has(key));
            converters.set(key, actor);
        }
    }
}
exports.loadConverters = loadConverters;
// This ought not be necessary, but we need some way to discover recursively
// typed symbols which do not have type nodes. See the `recursive` symbol in the variables test.
const seenTypeSymbols = new Set();
function convertType(context, typeOrNode, referenceTarget) {
    if (!typeOrNode) {
        return new models_1.IntrinsicType("any");
    }
    loadConverters();
    if ("kind" in typeOrNode) {
        const converter = converters.get(typeOrNode.kind);
        if (converter) {
            return converter.convert(context, typeOrNode, referenceTarget);
        }
        context.logger.warn(`Missing type node converter for kind ${typeOrNode.kind} (${ts.SyntaxKind[typeOrNode.kind]})`);
        return new models_1.UnknownType(typeOrNode.getText());
    }
    // IgnoreErrors is important, without it, we can't assert that we will get a node.
    const node = context.checker.typeToTypeNode(typeOrNode, void 0, ts.NodeBuilderFlags.IgnoreErrors);
    assert(node); // According to the TS source of typeToString, this is a bug if it does not hold.
    const symbol = typeOrNode.getSymbol();
    if (symbol) {
        if (node.kind !== ts.SyntaxKind.TypeReference &&
            seenTypeSymbols.has(symbol)) {
            const typeString = context.checker.typeToString(typeOrNode);
            context.logger.verbose(`Refusing to recurse when converting type: ${typeString}`);
            return new models_1.UnknownType(typeString);
        }
        seenTypeSymbols.add(symbol);
    }
    const converter = converters.get(node.kind);
    if (converter) {
        const result = converter.convertType(context, typeOrNode, node, referenceTarget);
        if (symbol)
            seenTypeSymbols.delete(symbol);
        return result;
    }
    context.logger.warn(`Missing type converter for type: ${context.checker.typeToString(typeOrNode)}`);
    return new models_1.UnknownType(context.checker.typeToString(typeOrNode));
}
exports.convertType = convertType;
const arrayConverter = {
    kind: [ts.SyntaxKind.ArrayType],
    convert(context, node, target) {
        return new models_1.ArrayType(convertType(context, node.elementType, target));
    },
    convertType(context, type, target) {
        const params = context.checker.getTypeArguments(type);
        assert(params.length === 1);
        return new models_1.ArrayType(convertType(context, params[0], target));
    },
};
const conditionalConverter = {
    kind: [ts.SyntaxKind.ConditionalType],
    convert(context, node, target) {
        return new models_1.ConditionalType(convertType(context, node.checkType, target), convertType(context, node.extendsType, target), convertType(context, node.trueType, target), convertType(context, node.falseType, target));
    },
    convertType(context, type, target) {
        return new models_1.ConditionalType(convertType(context, type.checkType, target), convertType(context, type.extendsType, target), convertType(context, type.resolvedTrueType, target), convertType(context, type.resolvedFalseType, target));
    },
};
const exprWithTypeArgsConverter = {
    kind: [ts.SyntaxKind.ExpressionWithTypeArguments],
    convert(context, node, target) {
        var _a, _b;
        const targetSymbol = context.getSymbolAtLocation(node.expression);
        // Mixins... we might not have a symbol here.
        if (!targetSymbol) {
            return convertType(context, context.checker.getTypeAtLocation(node), target);
        }
        const parameters = (_b = (_a = node.typeArguments) === null || _a === void 0 ? void 0 : _a.map((type) => convertType(context, type, target))) !== null && _b !== void 0 ? _b : [];
        const ref = new models_1.ReferenceType(targetSymbol.name, context.resolveAliasedSymbol(targetSymbol), context.project);
        ref.typeArguments = parameters;
        return ref;
    },
    convertType: requestBugReport,
};
const indexedAccessConverter = {
    kind: [ts.SyntaxKind.IndexedAccessType],
    convert(context, node, target) {
        return new models_1.IndexedAccessType(convertType(context, node.objectType, target), convertType(context, node.indexType, target));
    },
    convertType(context, type, target) {
        return new models_1.IndexedAccessType(convertType(context, type.objectType, target), convertType(context, type.indexType, target));
    },
};
const inferredConverter = {
    kind: [ts.SyntaxKind.InferType],
    convert(_context, node) {
        return new models_1.InferredType(node.typeParameter.getText());
    },
    convertType(_context, type) {
        return new models_1.InferredType(type.symbol.name);
    },
};
const intersectionConverter = {
    kind: [ts.SyntaxKind.IntersectionType],
    convert(context, node, target) {
        return new models_1.IntersectionType(node.types.map((type) => convertType(context, type, target)));
    },
    convertType(context, type, target) {
        return new models_1.IntersectionType(type.types.map((type) => convertType(context, type, target)));
    },
};
const keywordNames = {
    [ts.SyntaxKind.AnyKeyword]: "any",
    [ts.SyntaxKind.BigIntKeyword]: "bigint",
    [ts.SyntaxKind.BooleanKeyword]: "boolean",
    [ts.SyntaxKind.NeverKeyword]: "never",
    [ts.SyntaxKind.NumberKeyword]: "number",
    [ts.SyntaxKind.ObjectKeyword]: "object",
    [ts.SyntaxKind.StringKeyword]: "string",
    [ts.SyntaxKind.SymbolKeyword]: "symbol",
    [ts.SyntaxKind.UndefinedKeyword]: "undefined",
    [ts.SyntaxKind.UnknownKeyword]: "unknown",
    [ts.SyntaxKind.VoidKeyword]: "void",
    [ts.SyntaxKind.IntrinsicKeyword]: "intrinsic",
};
const keywordConverter = {
    kind: [
        ts.SyntaxKind.AnyKeyword,
        ts.SyntaxKind.BigIntKeyword,
        ts.SyntaxKind.BooleanKeyword,
        ts.SyntaxKind.NeverKeyword,
        ts.SyntaxKind.NumberKeyword,
        ts.SyntaxKind.ObjectKeyword,
        ts.SyntaxKind.StringKeyword,
        ts.SyntaxKind.SymbolKeyword,
        ts.SyntaxKind.UndefinedKeyword,
        ts.SyntaxKind.UnknownKeyword,
        ts.SyntaxKind.VoidKeyword,
    ],
    convert(_context, node) {
        return new models_1.IntrinsicType(keywordNames[node.kind]);
    },
    convertType(_context, _type, node) {
        return new models_1.IntrinsicType(keywordNames[node.kind]);
    },
};
const parensConverter = {
    kind: [ts.SyntaxKind.ParenthesizedType],
    convert(context, node, target) {
        return convertType(context, node.type, target);
    },
    // TS strips these out too... shouldn't run into this.
    convertType: requestBugReport,
};
const predicateConverter = {
    kind: [ts.SyntaxKind.TypePredicate],
    convert(context, node, target) {
        const name = ts.isThisTypeNode(node.parameterName)
            ? "this"
            : node.parameterName.getText();
        const asserts = !!node.assertsModifier;
        const targetType = node.type
            ? convertType(context, node.type, target)
            : void 0;
        return new models_1.PredicateType(name, asserts, targetType);
    },
    // Never inferred by TS 4.0, could potentially change in a future TS version.
    convertType: requestBugReport,
};
// This is a horrible thing... we're going to want to split this into converters
// for different types at some point.
const typeLiteralConverter = {
    kind: [ts.SyntaxKind.TypeLiteral, ts.SyntaxKind.FunctionType],
    convert(context, node) {
        var _a, _b;
        const symbol = (_a = context.getSymbolAtLocation(node)) !== null && _a !== void 0 ? _a : node.symbol;
        const type = context.getTypeAtLocation(node);
        if (!symbol || !type) {
            return new models_1.IntrinsicType("Object");
        }
        const reflection = new models_1.DeclarationReflection("__type", models_1.ReflectionKind.TypeLiteral, context.scope);
        context.registerReflection(reflection, symbol);
        context.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, reflection, node);
        for (const prop of context.checker.getPropertiesOfType(type)) {
            symbols_1.convertSymbol(context.withScope(reflection), prop);
        }
        for (const signature of type.getCallSignatures()) {
            (_b = reflection.signatures) !== null && _b !== void 0 ? _b : (reflection.signatures = []);
            reflection.signatures.push(signature_1.createSignature(context.withScope(reflection), models_1.ReflectionKind.CallSignature, signature));
        }
        return new models_1.ReflectionType(reflection);
    },
    convertType(context, type) {
        var _a;
        if (!type.symbol) {
            return new models_1.IntrinsicType("Object");
        }
        const reflection = new models_1.DeclarationReflection("__type", models_1.ReflectionKind.TypeLiteral, context.scope);
        context.registerReflection(reflection, type.symbol);
        context.trigger(converter_events_1.ConverterEvents.CREATE_DECLARATION, reflection);
        for (const prop of context.checker.getPropertiesOfType(type)) {
            symbols_1.convertSymbol(context.withScope(reflection), prop);
        }
        for (const signature of type.getCallSignatures()) {
            (_a = reflection.signatures) !== null && _a !== void 0 ? _a : (reflection.signatures = []);
            reflection.signatures.push(signature_1.createSignature(context.withScope(reflection), models_1.ReflectionKind.CallSignature, signature));
        }
        return new models_1.ReflectionType(reflection);
    },
};
const queryConverter = {
    kind: [ts.SyntaxKind.TypeQuery],
    convert(context, node) {
        const querySymbol = context.expectSymbolAtLocation(node.exprName);
        return new models_1.QueryType(new models_1.ReferenceType(node.exprName.getText(), context.resolveAliasedSymbol(querySymbol), context.project));
    },
    convertType(context, type) {
        const symbol = type.getSymbol();
        assert(symbol, `Query type failed to get a symbol for: ${context.checker.typeToString(type)}. This is a bug.`);
        return new models_1.QueryType(new models_1.ReferenceType(symbol.name, context.resolveAliasedSymbol(symbol), context.project));
    },
};
const referenceConverter = {
    kind: [ts.SyntaxKind.TypeReference],
    convert(context, node, target) {
        var _a;
        const symbol = context.expectSymbolAtLocation(node.typeName);
        // We might need to resolve this type (e.g a parent class has a property with a generic type)
        // but we are in a child type, so the generic type should be resolved to a concrete one.
        if (target && symbol.flags & ts.SymbolFlags.TypeParameter) {
            const resolvedType = context.checker.getTypeOfSymbolAtLocation(symbol, target);
            if (!isErrorType(resolvedType) && !resolvedType.isTypeParameter()) {
                return convertType(context, resolvedType, target);
            }
        }
        const name = node.typeName.getText();
        const type = new models_1.ReferenceType(name, context.resolveAliasedSymbol(symbol), context.project);
        type.typeArguments = (_a = node.typeArguments) === null || _a === void 0 ? void 0 : _a.map((type) => convertType(context, type, target));
        return type;
    },
    convertType(context, type, target) {
        var _a, _b;
        const symbol = (_a = type.aliasSymbol) !== null && _a !== void 0 ? _a : type.getSymbol();
        if (!symbol) {
            // If we get in here, the user is doing something bad. Probably using mixins.
            const broken = new models_1.UnknownType(context.checker.typeToString(type));
            context.logger.warn(`Bad reference type: ${broken.name}`);
            return broken;
        }
        const ref = new models_1.ReferenceType(symbol.name, context.resolveAliasedSymbol(symbol), context.project);
        ref.typeArguments = (_b = type.aliasTypeArguments) === null || _b === void 0 ? void 0 : _b.map((ref) => convertType(context, ref, target));
        return ref;
    },
};
const namedTupleMemberConverter = {
    kind: [ts.SyntaxKind.NamedTupleMember],
    convert(context, node, target) {
        const innerType = convertType(context, node.type, target);
        return new models_1.NamedTupleMember(node.name.getText(), !!node.questionToken, innerType);
    },
    // This ought to be impossible.
    convertType: requestBugReport,
};
// { -readonly [K in string]-?: number}
//   ^ readonlyToken
//              ^ typeParameter
//                   ^^^^^^ typeParameter.constraint
//                          ^ questionToken
//                              ^^^^^^ type
const mappedConverter = {
    kind: [ts.SyntaxKind.MappedType],
    convert(context, node, target) {
        var _a, _b;
        const optionalModifier = kindToModifier((_a = node.questionToken) === null || _a === void 0 ? void 0 : _a.kind);
        const templateType = convertType(context, node.type, target);
        return new models_1.MappedType(node.typeParameter.name.text, convertType(context, node.typeParameter.constraint, target), optionalModifier === "+"
            ? removeUndefined(templateType)
            : templateType, kindToModifier((_b = node.readonlyToken) === null || _b === void 0 ? void 0 : _b.kind), optionalModifier, node.nameType ? convertType(context, node.nameType, target) : void 0);
    },
    convertType(context, type, node, target) {
        var _a, _b, _c;
        // This can happen if a generic function does not have a return type annotated.
        const optionalModifier = kindToModifier((_a = node.questionToken) === null || _a === void 0 ? void 0 : _a.kind);
        const templateType = convertType(context, type.templateType, target);
        return new models_1.MappedType((_b = type.typeParameter.symbol) === null || _b === void 0 ? void 0 : _b.name, convertType(context, type.typeParameter.getConstraint(), target), optionalModifier === "+"
            ? removeUndefined(templateType)
            : templateType, kindToModifier((_c = node.readonlyToken) === null || _c === void 0 ? void 0 : _c.kind), optionalModifier, type.nameType ? convertType(context, type.nameType, target) : void 0);
    },
};
const literalTypeConverter = {
    kind: [ts.SyntaxKind.LiteralType],
    convert(context, node) {
        switch (node.literal.kind) {
            case ts.SyntaxKind.TrueKeyword:
            case ts.SyntaxKind.FalseKeyword:
                return new models_1.LiteralType(node.literal.kind === ts.SyntaxKind.TrueKeyword);
            case ts.SyntaxKind.StringLiteral:
                return new models_1.LiteralType(node.literal.text);
            case ts.SyntaxKind.NumericLiteral:
                return new models_1.LiteralType(Number(node.literal.text));
            case ts.SyntaxKind.NullKeyword:
                return new models_1.LiteralType(null);
            case ts.SyntaxKind.PrefixUnaryExpression: {
                const operand = node.literal
                    .operand;
                switch (operand.kind) {
                    case ts.SyntaxKind.NumericLiteral:
                        return new models_1.LiteralType(Number(node.literal.getText()));
                    case ts.SyntaxKind.BigIntLiteral:
                        return new models_1.LiteralType(BigInt(node.literal.getText().replace("n", "")));
                    default:
                        return requestBugReport(context, node.literal);
                }
            }
            case ts.SyntaxKind.BigIntLiteral:
                return new models_1.LiteralType(BigInt(node.literal.getText().replace("n", "")));
        }
        return requestBugReport(context, node.literal);
    },
    convertType(context, type, node) {
        switch (node.literal.kind) {
            case ts.SyntaxKind.StringLiteral:
                return new models_1.LiteralType(node.literal.text);
            case ts.SyntaxKind.NumericLiteral:
                return new models_1.LiteralType(+node.literal.text);
            case ts.SyntaxKind.TrueKeyword:
            case ts.SyntaxKind.FalseKeyword:
                return new models_1.LiteralType(node.literal.kind === ts.SyntaxKind.TrueKeyword);
            case ts.SyntaxKind.NullKeyword:
                return new models_1.LiteralType(null);
        }
        if (typeof type.value === "object") {
            return new models_1.LiteralType(BigInt(`${type.value.negative ? "-" : ""}${type.value.base10Value}`));
        }
        return requestBugReport(context, type);
    },
};
const templateLiteralConverter = {
    kind: [ts.SyntaxKind.TemplateLiteralType],
    convert(context, node, target) {
        return new template_literal_1.TemplateLiteralType(node.head.text, node.templateSpans.map((span) => {
            return [
                convertType(context, span.type, target),
                span.literal.text,
            ];
        }));
    },
    convertType(context, type, _node, target) {
        assert(type.texts.length === type.types.length + 1);
        const parts = [];
        for (const [a, b] of array_1.zip(type.types, type.texts.slice(1))) {
            parts.push([convertType(context, a, target), b]);
        }
        return new template_literal_1.TemplateLiteralType(type.texts[0], parts);
    },
};
const thisConverter = {
    kind: [ts.SyntaxKind.ThisType],
    convert() {
        return new models_1.IntrinsicType("this");
    },
    convertType() {
        return new models_1.IntrinsicType("this");
    },
};
const tupleConverter = {
    kind: [ts.SyntaxKind.TupleType],
    convert(context, node, target) {
        var _a;
        // TS 3.9 support
        const elementTypes = (_a = node.elements) !== null && _a !== void 0 ? _a : node.elementTypes;
        const elements = elementTypes.map((node) => convertType(context, node, target));
        return new models_1.TupleType(elements);
    },
    convertType(context, type, node, target) {
        var _a;
        let elements = (_a = type.typeArguments) === null || _a === void 0 ? void 0 : _a.map((type) => convertType(context, type, target));
        if (node.elements.every(ts.isNamedTupleMember)) {
            const namedMembers = node.elements;
            elements = elements === null || elements === void 0 ? void 0 : elements.map((el, i) => new models_1.NamedTupleMember(namedMembers[i].name.text, !!namedMembers[i].questionToken, removeUndefined(el)));
        }
        return new models_1.TupleType(elements !== null && elements !== void 0 ? elements : []);
    },
};
const supportedOperatorNames = {
    [ts.SyntaxKind.KeyOfKeyword]: "keyof",
    [ts.SyntaxKind.UniqueKeyword]: "unique",
    [ts.SyntaxKind.ReadonlyKeyword]: "readonly",
};
const typeOperatorConverter = {
    kind: [ts.SyntaxKind.TypeOperator],
    convert(context, node, target) {
        return new models_1.TypeOperatorType(convertType(context, node.type, target), supportedOperatorNames[node.operator]);
    },
    convertType(context, type, node, target) {
        // readonly is only valid on array and tuple literal types.
        if (node.operator === ts.SyntaxKind.ReadonlyKeyword) {
            assert(isObjectType(type));
            const args = context.checker
                .getTypeArguments(type)
                .map((type) => convertType(context, type, target));
            const inner = type.objectFlags & ts.ObjectFlags.Tuple
                ? new models_1.TupleType(args)
                : new models_1.ArrayType(args[0]);
            return new models_1.TypeOperatorType(inner, "readonly");
        }
        // keyof will only show up with generic functions, otherwise it gets eagerly
        // resolved to a union of strings.
        if (node.operator === ts.SyntaxKind.KeyOfKeyword) {
            // There's probably an interface for this somewhere... I couldn't find it.
            const targetType = type.type;
            return new models_1.TypeOperatorType(convertType(context, targetType, target), "keyof");
        }
        // TS drops `unique` in `unique symbol` everywhere. If someone used it, we ought
        // to have a type node. This shouldn't ever happen.
        return requestBugReport(context, type);
    },
};
const unionConverter = {
    kind: [ts.SyntaxKind.UnionType],
    convert(context, node, target) {
        return new models_1.UnionType(node.types.map((type) => convertType(context, type, target)));
    },
    convertType(context, type, target) {
        return new models_1.UnionType(type.types.map((type) => convertType(context, type, target)));
    },
};
function requestBugReport(context, nodeOrType) {
    if ("kind" in nodeOrType) {
        const kindName = ts.SyntaxKind[nodeOrType.kind];
        const { line, character } = ts.getLineAndCharacterOfPosition(nodeOrType.getSourceFile(), nodeOrType.pos);
        context.logger.warn(`Failed to convert type node with kind: ${kindName} and text ${nodeOrType.getText()}. Please report a bug.\n\t` +
            `${nodeOrType.getSourceFile().fileName}:${line + 1}:${character}`);
        return new models_1.UnknownType(nodeOrType.getText());
    }
    else {
        const typeString = context.checker.typeToString(nodeOrType);
        context.logger.warn(`Failed to convert type: ${typeString}. Please report a bug.`);
        return new models_1.UnknownType(typeString);
    }
}
function isObjectType(type) {
    return typeof type.objectFlags === "number";
}
function kindToModifier(kind) {
    switch (kind) {
        case ts.SyntaxKind.ReadonlyKeyword:
        case ts.SyntaxKind.QuestionToken:
        case ts.SyntaxKind.PlusToken:
            return "+";
        case ts.SyntaxKind.MinusToken:
            return "-";
        default:
            return undefined;
    }
}
function removeUndefined(type) {
    if (type instanceof models_1.UnionType) {
        const types = type.types.filter((t) => !t.equals(new models_1.IntrinsicType("undefined")));
        if (types.length === 1) {
            return types[0];
        }
        type.types = types;
        return type;
    }
    return type;
}
function isErrorType(type) {
    return type.intrinsicName === "error";
}
//# sourceMappingURL=types.js.map