"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Comment = void 0;
const tag_1 = require("./tag");
/**
 * A model that represents a javadoc comment.
 *
 * Instances of this model are created by the [[CommentHandler]]. You can retrieve comments
 * through the [[BaseReflection.comment]] property.
 */
class Comment {
    /**
     * Creates a new Comment instance.
     */
    constructor(shortText, text) {
        this.shortText = shortText || "";
        this.text = text || "";
    }
    /**
     * Has this comment a visible component?
     *
     * @returns TRUE when this comment has a visible component.
     */
    hasVisibleComponent() {
        return !!this.shortText || !!this.text || !!this.tags;
    }
    /**
     * Test whether this comment contains a tag with the given name.
     *
     * @param tagName  The name of the tag to look for.
     * @returns TRUE when this comment contains a tag with the given name, otherwise FALSE.
     */
    hasTag(tagName) {
        if (!this.tags) {
            return false;
        }
        for (let i = 0, c = this.tags.length; i < c; i++) {
            if (this.tags[i].tagName === tagName) {
                return true;
            }
        }
        return false;
    }
    /**
     * Return the first tag with the given name.
     *
     * You can optionally pass a parameter name that should be searched to.
     *
     * @param tagName  The name of the tag to look for.
     * @param paramName  An optional parameter name to look for.
     * @returns The found tag or undefined.
     */
    getTag(tagName, paramName) {
        return (this.tags || []).find((tag) => {
            return (tag.tagName === tagName &&
                (paramName === void 0 || tag.paramName === paramName));
        });
    }
    /**
     * Removes all tags with the given tag name from teh comment.
     * @param tagName
     */
    removeTags(tagName) {
        var _a;
        if (!this.tags) {
            return;
        }
        let i = 0, c = (_a = this.tags.length) !== null && _a !== void 0 ? _a : 0;
        while (i < c) {
            if (this.tags[i].tagName === tagName) {
                this.tags.splice(i, 1);
                c--;
            }
            else {
                i++;
            }
        }
    }
    /**
     * Copy the data of the given comment into this comment.
     *
     * @param comment
     */
    copyFrom(comment) {
        this.shortText = comment.shortText;
        this.text = comment.text;
        this.returns = comment.returns;
        this.tags = comment.tags
            ? comment.tags.map((tag) => new tag_1.CommentTag(tag.tagName, tag.paramName, tag.text))
            : undefined;
    }
}
exports.Comment = Comment;
//# sourceMappingURL=comment.js.map